<script setup>
import AppLayoutWater from "@/Layouts/AppLayoutWater.vue";
import Welcome from "@/Jetstream/Welcome.vue";
import mapboxgl from "mapbox-gl";
import "mapbox-gl/dist/mapbox-gl.css";
import VuePapaParse from "vue-papa-parse";
import { conditionalExpression, templateElement } from "@babel/types";
import { Link } from '@inertiajs/inertia-vue3';

</script>
<style>
*,
::before,
::after {
  --tw-ring-color: rgb(59 130 246 / 0) !important;
}

.map-overlay {
  position: absolute;
  width: 30px;
  top: 4.5rem;
  left: 1rem;
  padding: 0px;
}

.map-overlay .map-overlay-inner {
  background-color: rgba(0, 42, 80, 0);
  /*box-shadow: 0 1px 2px rgba(0, 0, 0, 0.2);*/
  border-radius: 8px;
  padding: 0px;
  margin-bottom: 10px;
  height: 60vh;
}

/**/
#container {
  max-width: 660px;
  margin: auto;
  height: 400px;
  margin: 0 auto;
}

.percents {
  font-weight: bold;
  font-size: 22px;
}

.divide {
  width: 100%;
  margin: 0px;
  border: 0.5px solid black;
}
</style>
<template>
  <AppLayoutWater title="Global View">
    <div v-if="this.screen_width >= 768" class="baltoro-container-2 overflow-x-hidden w-48 h-fit text-white-snow box-border z-50 ml-4 mt-16 left-0 top-0 fixed"
      id="screendevices">
      <input type="text" v-on:keyup="searchfilter()" :placeholder="$t('general.Search')"
        class="focus-visible:outline-0 bg-cyan-opa text-md text-center text-white-snow font-normal w-48" id="searchbar" />
      <ul class="text-xs text-center text-gray-light max-h-48 h-fit overflow-y-scroll" id="device_list"></ul>
    </div>

    <div id="legend"
      class="baltoro-container-2 w-fit h-fit box-border z-50 ml-4 mb-11 left-0 bottom-0 fixed flex flex-col py-1 px-2 text-gray-light text-sm">
    </div>

    <!-- START RIGHT SIDEMENU -->
    <div id="sidemenu" style="height: calc(100vh - 4.75rem) !important" v-if="this.screen_width >= 768"
      class="cursor-default bg-transparent content-start grid grid-cols-2 gap-4 box-border duration-200 overflow-y-scroll overflow-x-hidden over z-50 pb-7 right-0 top-13 w-0 fixed float-left">
      <div class="col-span-2" id="sidemenu_content_before"></div>

      <!-- CHARTS SEPARADOS-->
      <div v-for="(Name, index) in this.charts" :key="index" :id="'chart_box_' + index"
        class="baltoro-container-2 w-full h-fit duration-200" v-bind:class="{
            'hidden': this.charts[index] == undefined,
            'col-span-2 pr-4': !this.charts_size.includes(index),
            'col-span-1': this.charts_size.includes(index),
          }">
        <div class="border-0" v-bind:class="{
            'flex flex-col items-center': this.charts_size.includes(index),
          }">
          <p v-if="this.charts[index] != undefined" class="text-xs text-center text-cyan-text font-normal my-1">
            {{ this.charts[index].name }}
          </p>
          <div :id="'chart_' + index" class="float-left" v-bind:class="{
              'w-full h-[3.6rem]':
                !this.charts_size.includes(index) &&
                !this.charts_text.includes(index) &&
                this.charts_height.includes(index),
              'w-full h-32':
                !this.charts_size.includes(index) &&
                !this.charts_text.includes(index) &&
                !this.charts_height.includes(index),
              'h-32':
                this.charts_size.includes(index) &&
                !this.charts_text.includes(index),
              '':
                this.charts_text.includes(index),
            }"></div>
          <div :id="'chart_text_' + index"></div>
        </div>
      </div>

      <div class="col-span-2" id="sidemenu_content_after"></div>
    </div>
    <!-- END RIGHT SIDEMENU -->

    <!-- map -->
    <div style="height: calc(100vh - 4.75rem)" id="map" />

    <!-- overlay -->
    <div class="map-overlay">
      <div class="map-overlay-inner"></div>
    </div>

    <div v-for="(gateway, index) in this.gateways" :key="index" class="h-0 w-0 invisible">
      <Link :id="'LinkProfile_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('meterProfile', { id: gateway.id })"></Link>
      <Link :id="'LinkReadings_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('Water_Readings', { id: gateway.id })"></Link>
      <Link :id="'LinkPressureTransients_'+gateway.id" class="h-0 w-0 invisible" onclick="window.emitter.emit('changeLoaderStatus', true, 'orbit')" :href="route('Water_PressureTransients', { id: gateway.id })"></Link>
    </div>
  </AppLayoutWater>
</template>

<script>
export default {
  data() {
    return {
      screen_width: 0,
      Chart: {},
      geojsonobj: {},
      Chart: {},
      id_open: undefined,
      charts: new Array(10),
      charts_size: new Array(),
      charts_height: new Array(),
      charts_text: new Array(),
      default_charts: new Array(10),
      default_charts_size: new Array(),
      default_charts_height: new Array(),
      default_charts_text: new Array(),
      showlist: false,
      week_day: new Date().getDay(),
      select_mode: 0,
      map: undefined,

      getLastData: (id) => {
        return axios({
          method: "get",
          url:
            window.location.origin +
            "/api/gateway/data_summary_last/" +
            id +
            "/" +
            this.user.id +
            "/0/" +
            this.user.timezone_offset,
          headers: {
            Accept: "application/json",
            "Content-Type": "application/json",
            Authorization: "Bearer " + this.user.token,
          },
        })
          .then((response) => {
            return response.data;
          })
          .catch((error) => {
            window.emitter.emit('changeLoaderStatus', false, 'orbit')

            if(error.response && error.response.status && error.response.status == 429){
              window.alert( this.$t('text.Api attempts'));
            }else{
              return "Failed!", error;
            }
          });
      },

      getData: (id) => {
        window.emitter.emit('changeLoaderStatus', true, 'orbit')
        return axios({
          method: "get",
          url: window.location.origin + '/api/gateway/data_summary_day/' + id + '/0/' + this.user.id + '/1/' + this.user.timezone_offset,
          headers: {
            Accept: "application/json",
            "Content-Type": "application/json",
            Authorization: "Bearer " + this.user.token,
          },
        })
          .then((response) => {
            window.emitter.emit('changeLoaderStatus', false, 'orbit')
            return response.data;
          })
          .catch((error) => {
                window.emitter.emit('changeLoaderStatus', false, 'orbit')

                if(error.response && error.response.status && error.response.status == 429){
                  window.alert( this.$t('text.Api attempts'));
                }else{

                  return "Failed!", error;
                }
              });      
            },

      formatDate: (date, utc, short) => {
        date = new Date(date);

        // utc en 0 para formatear hora local del usuario, 1 para formatear UTC
        if (utc == 0) {
          date.setHours(date.getHours() + this.user.timezone_offset);
        } else if (utc == -1) {
          date.setHours(date.getHours() - this.user.timezone_offset);
        }

        if (!short) {
          return (
            date.getFullYear() +
            "-" +
            ((date.getMonth() + 1 + "").length < 2
              ? "0" + (date.getMonth() + 1)
              : date.getMonth() + 1) +
            "-" +
            ((date.getDate() + "").length < 2
              ? "0" + date.getDate()
              : date.getDate()) +
            " " +
            ((date.getHours() + "").length < 2
              ? "0" + date.getHours()
              : date.getHours()) +
            ":" +
            ((date.getMinutes() + "").length < 2
              ? "0" + date.getMinutes()
              : date.getMinutes()) +
            ":" +
            ((date.getSeconds() + "").length < 2
              ? "0" + date.getSeconds()
              : date.getSeconds())
          );
        } else {
          return (
            ((date.getMonth() + 1 + "").length < 2
              ? "0" + (date.getMonth() + 1)
              : date.getMonth() + 1) +
            "-" +
            ((date.getDate() + "").length < 2
              ? "0" + date.getDate()
              : date.getDate()));
        }
      },

      formatTime: (date_state_change, diff, short) => {

        let duration_ms;
        if (diff == true) {
          let date = new Date(date_state_change);

          if (date.getHours() == 0 && date.getMinutes() == 0 && date.getSeconds() == 0) {
            return 'yesterday'; // Cambiarlo
          }

          duration_ms = new Date() - date;
        } else {
          duration_ms = date_state_change * 1000;
        }

        const diff_D = Math.floor(duration_ms / (1000 * 60 * 60 * 24));
        const diff_H = Math.floor(
          (duration_ms % (1000 * 60 * 60 * 24)) / (1000 * 60 * 60)
        );
        const diff_M = Math.floor(
          (duration_ms % (1000 * 60 * 60)) / (1000 * 60)
        );

        let duration = "";

        if (this.user.language == "es" && diff) {
          duration = this.$t("general.for") + " ";
        }

        if (diff_D > 0) {
          duration += this.$t("general.morethan") + " " + diff_D + " ";
          if (diff_D > 1) {
            duration += this.$t("general.days");
          } else {
            duration += this.$t("general.day");
          }
        } else {
          if (diff_H > 0) {
            duration += diff_H;
            if (diff_M > 0) {
              duration += this.$t("general.h");
            } else if (diff_H > 1) {
              duration += " " + this.$t("general.hours");
            } else {
              duration += " " + this.$t("general.hour");
            }

            /*if(diff_M != 0){
                duration += ' '
              }*/

            if (diff_M != 0) {
              duration += " " + this.$t("general.and") + " ";
            }
          }

          if (diff_M > 0) {
            duration += diff_M;
            if (diff_H != 0 && diff_M > 1) {
              duration += this.$t("general.mins");
            } else if (diff_H != 0 && diff_M == 1) {
              duration += this.$t("general.min");
            } else if (diff_M > 1) {
              duration += " " + this.$t("general.minutes");
            } else {
              duration += " " + this.$t("general.minute");
            }
          } else if (short) {
            return this.$t("general.now");
          }
        }

        if (this.user.language == "en" && diff) {
          duration += " " + this.$t("general.ago");
        }

        return duration;
      },
    };
  },

  props: ["gateways", "group_sector", "group_service", "location", "alerts_log", "data", "user"],

  beforeCreate() {
    this.$i18n.locale = this.user.language;
    localStorage.setItem("Language", this.$i18n.locale);
    localStorage.setItem("Back", "map");

    localStorage.setItem("Alerts", 0);

    for (var index = 0; index < this.alerts_log.length; index++) {
      if (this.alerts_log[index].read == 0) {
        localStorage.setItem("Alerts", +localStorage.getItem("Alerts") + 1);
      }
    }

    if (+localStorage.getItem("Alerts") > 9) {
      localStorage.setItem("Alerts", "9+");
    }

  },
  
  beforeMount() {
      this.screen_width = window.screen.width;
  },


  mounted() {
        window.emitter.emit('changeLoaderStatus', false, 'orbit')

    let formatDate = this.formatDate;

    this.Chart = {
      chart: {
        backgroundColor: "transparent",
        spacingTop: 15,
      },
      accessibility: {
        enabled: false,
      },
      title: {
        text: "",
      },
      xAxis: {
        visible: true,
        labels: {
          style: {
            color: "rgba(200, 200, 200, 0.9)",
            opacity: 0.7,
          },
        },
        type: "datetime",
        dateTimeLabelFormats: {
          hour: "%H:%M",
          day: "%m-%d",
          week: "%m-%d",
          month: "%Y-%m",
          year: "%Y",
        },
      },
      yAxis: {
        title: {
          text: "",
        },
        labels: {
          style: {
            color: "rgba(200, 200, 200, 0.9)",
          },
        },
        visible: true,
      },
      exporting: {
        enabled: false,
      },
      credits: {
        enabled: false,
      },
      legend: {
        enabled: false,
      },
      tooltip: {
        shared: true,
        split: false,
        backgroundColor: "rgba(7,82,103,0.95)",
        borderRadius: 12,
        borderWidth: 1,
        borderColor: "rgba(250, 250, 250, 0.65)",
        padding: 6,

        formatter: function () {
          var s =
            "<b>" +
            '<p style="font-size:12px; line-height: 12px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' +
            formatDate(this.x, 1, false) +
            "</p><br/>";
          $.each(this.points, function () {
            s +=
              '<p style="font-size:12px; line-height: 12px; font-weight: 400; color:' +
              this.series.color +
              '">' +
              this.series.name +
              ' </p><p style="font-size:12px; line-height: 12px; font-weight: 400; color:' +
              this.series.color +
              '">' +
              this.y +
              " " +
              this.series.userOptions.tooltip.valueSuffix +
              "</p><br/>";
          });

          return s;
        },
      },
    };

    for (let index in this.data) {

      if (typeof this.data[index]['group']) {
        this.setGroupData(index);
      }
    }

    this.group_service.forEach((service, index) => {
      this.geojsonobj[service.name] = {};
    });

    mapboxgl.accessToken =
      "pk.eyJ1IjoiZGF0YWtvcnVtLWRldmVsb3AiLCJhIjoiY2w0ODhqOXc2MDR1MjNjbzJ2YnZkdjNhYSJ9.pVPMnmkwOuF4DonlbNkkEw";

    const map = new mapboxgl.Map({
      attributionControl: false,
      container: "map",
      style: "mapbox://styles/datakorum-develop/cl4cu370w000v15tal53sxn9v",
      center: [this.location.GPS_lng, this.location.GPS_lat],
      zoom: 17,
      bearing: 0,
      maxZoom: 20,
      minZoom: 5,
      pitch: this.screen_width >= 768 ? 67.5 : 10,
    });

    let geojsonobj;
    var leaverdevices = 1,
        leaverpopup = 1;
    var popup, popups, coordinates, description, service_name;

    map.on("load", () => {

      map.loadImage('assets/images/markers/distributionLang.png', (error, image) => {
        if (error) throw error;
        map.addImage('sectorial_marker', image);
      });

      map.loadImage('assets/images/markers/distributionLang_alert.png', (error, image) => {
        if (error) throw error;
        map.addImage('sectorial_marker_alert', image);
      });


      map.loadImage('assets/images/markers/domiciliaryLang.png', (error, image) => {
        if (error) throw error;
        map.addImage('domiciliary_marker', image);
      });

      map.loadImage('assets/images/markers/domiciliaryLang_alert.png', (error, image) => {
        if (error) throw error;
        map.addImage('domiciliary_marker_alert', image);
      });


      map.loadImage('assets/images/markers/pressureLang.png', (error, image) => {
        if (error) throw error;
        map.addImage('pressure_marker', image);
      });

      map.loadImage('assets/images/markers/pressureLang_alert.png', (error, image) => {
        if (error) throw error;
        map.addImage('pressure_marker_alert', image);
      });


      map.loadImage('assets/images/markers/waterQualityLang.png', (error, image) => {
        if (error) throw error;
        map.addImage('waterQuality_marker', image);
      });

      map.loadImage('assets/images/markers/waterQualityLang_alert.png', (error, image) => {
        if (error) throw error;
        map.addImage('waterQuality_marker_alert', image);
      });


      map.loadImage('assets/images/markers/waterLevelLang.png', (error, image) => {
        if (error) throw error;
        map.addImage('waterLevel_marker', image);
      });

      map.loadImage('assets/images/markers/waterLevelLang_alert.png', (error, image) => {
        if (error) throw error;
        map.addImage('waterLevel_marker_alert', image);
      });


      map.loadImage('assets/images/markers/clusterLang.png', (error, image) => {
        if (error) throw error;
        map.addImage('cluster_marker', image);
      });

      map.loadImage('assets/images/markers/clusterLang_alert.png', (error, image) => {
        if (error) throw error;
        map.addImage('cluster_marker_alert', image);
      });

      this.group_service.forEach((service, index) => {
        geojsonobj = {};
        var newFeature = {};
        geojsonobj["type"] = "FeatureCollection";
        geojsonobj["features"] = [];

        service_name = service.name;

        this.gateways.forEach((gateway) => {

          if (service.name == gateway.service_name) {    
            
            if (gateway.type_name.indexOf("PULSES") != -1) {
              description = 
              '<div class="content-between m-0 cursor-default text-xs">' +
              '<div class="bg-cyan-opa px-2 mb-2 mt-1 text-center rounded-lg flex justify-center items-end"><p id="popupLastflow_'+ gateway.id +'" class="text-2xl font-sevensegment text-white-snow">...</p><p class="text-white-snow pb-1">&nbsp;m³/h</p></div>' +
              '<div class="border-t border-gray-medium mb-1"></div>' +
              '<div class="flex justify-between text-cyan-text"><p>' + this.$t("general.Name") + ' &nbsp</p><p class="text-gray-light">' + gateway.name + '</p></div>' +
              '<div class="flex justify-between text-cyan-text"><p>' + this.$t("general.Id") + ' &nbsp</p><p class="text-gray-light">' + gateway.id + '</p></div>' +
              '<div class="flex justify-between text-cyan-text"><p>' + this.$t("general.Type") + ' &nbsp</p><p class="text-gray-light">' + gateway.type_name + '</p></div>' +
              //'<div class="flex justify-between text-cyan-text"><p>Sectors: &nbsp</p><p id="popupSectors" class="text-gray-light"> + </p></div>'+
              '<div class="flex justify-between text-cyan-text mb-2"><p>' + this.$t("general.Date") + ' &nbsp</p><p id="popupDate_'+ gateway.id +'" class="text-gray-light">...</p></div>' +
              '<div class="grid grid-cols-2 text-center text-gray-light gap-2">' +
              '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnReadings">' +
              '<i class="fa-solid fa-chart-column" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
              '&nbsp;' + this.$t("general.Readings") + '' +
              '</div>' +
              '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnProfile">' +
              '<i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
              '&nbsp;' + this.$t("general.Profile") + '' +
              '</div>'/* +
              '</div>' +
              '</div>';*/
            } else if (gateway.type_name.indexOf("PARK") != -1) {

            } else if (gateway.type_name.indexOf("MODBUS") != -1 || gateway.MODBUS_device_type != "") {
              description =
                '<div class="content-between m-0 cursor-default text-xs">' +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Name") +
                ' &nbsp</p><p class="text-gray-light">' +
                gateway.name +
                "</p></div>" +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Id") +
                ' &nbsp</p><p class="text-gray-light">' +
                gateway.id +
                "</p></div>" +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Group") +
                ' &nbsp</p><p class="text-gray-light">' +
                gateway.group_name +
                "</p></div>" +
                '<div class="flex justify-between text-cyan-text"><p>' +
                this.$t("general.Type") +
                ' &nbsp</p><p class="text-gray-light">' +
                gateway.type_name +
                "</p></div>";

            description +=
              '<div class="flex justify-between text-cyan-text"><p>' +
              this.$t("general.Last sent") +
              ' &nbsp</p><p id="popupDate_'+ gateway.id +'" class="text-gray-light"></p></div>' +
              '<div class="grid grid-cols-2 text-center text-gray-light gap-2 mt-2">' +
              '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnReadings">' +
              '<i class="fa-solid fa-chart-column" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
              "&nbsp;" +
              this.$t("general.Readings") +
              "" +
              "</div>" +
              '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnProfile">' +
              '<i class="fa-solid fa-gears" style="font-size:0.75rem; color:#FFFAFA;"></i>' +
              "&nbsp;" +
              this.$t("general.Profile") +
              "" +
              "</div>";

              if(gateway.MODBUS_device_type == 6){

                description += 
                  '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnState">' +
                  this.$t("general.State") +
                  "" +
                  "</div>" +
                  '<div class="rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnParameters">' +
                  this.$t("general.Parameters") +
                  "" +
                  "</div>";
              }

              //description += '</div>';
            
            }

            if(gateway.pres_transients == true){

              description += 
              '<div class="col-span-2 rounded-md bg-cyan-opa p-1 cursor-pointer flex items-center justify-center" id="btnPressureTransients">' +
                this.$t("general.PressureTransients") +
                "" +
              "</div>";
            }

            description += '</div>';

            newFeature = {
              type: "Feature",
              properties: {
                name: gateway.name,
                type_name: gateway.type_name,
                group_name: gateway.group_name,
                id: gateway.id,
                idtype: gateway.idtype,
                service_name: service_name,
                description: description,
                insidescreen: "",
              },
              geometry: {
                type: "Point",
                coordinates: [gateway.GPS_lng, gateway.GPS_lat],
              },
            };

            for (var index = 0; index < this.alerts_log.length; index++) {
              if (
                this.alerts_log[index].read == 0 &&
                newFeature.properties.id == this.alerts_log[index].iddevice
              ) {
                newFeature.properties.service_name =
                  service_name + "_alert";
              }
            }

            geojsonobj["features"].push(newFeature);
          }
        });

        if(geojsonobj.features.length != 0)
          this.geojsonobj[service_name] = geojsonobj;

        // Add data to map
        map.addSource(service.name, {
          type: "geojson",
          data: geojsonobj,
          cluster: true,
          clusterMaxZoom: 40,
          clusterRadius: 40,
        });

        // Layer individual device

        map.addLayer({
          id: service.name,
          type: "symbol",
          source: service.name,
          filter: ["!", ["has", "point_count"]],
          layout: {
            "icon-image": [
              "match",
              ["get", "service_name"],              

              'Distribución_alert', 'sectorial_marker_alert',

              'Domiciliario_alert', 'domiciliary_marker_alert',

              'Control de presión_alert', 'pressure_marker_alert',

              'Calidad de agua_alert', 'waterQuality_marker_alert',

              'waterLevelLang_alert', 'waterLevelLang_marker_alert',

              'Distribución', 'sectorial_marker',

              'Domiciliario', 'domiciliary_marker',

              'Control de presión', 'pressure_marker',

              'Calidad de agua', 'waterQuality_marker',

              'waterLevelLang', 'waterLevelLang_marker',

              'domiciliary_marker'

            ],
            "icon-anchor": "bottom",
            "icon-size": 0.9,
            visibility: "visible",
          },
        });

        // CLICK THE DEVICE
        map.on("click", service.name, (e) => {
          this.week_day = new Date().getDay();

          coordinates = this.coordinatesFromMarker(e);
          this.show(
            e,
            e.features[0].properties.id,
            map
          );

          this.id_open = e.features[0].properties.id;
          
          if (this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0") {
            //cuando esté abierto
            const description = e.features[0].properties.description;

            popup = new mapboxgl.Popup()
              .setLngLat(coordinates)
              .setHTML(description);
          }

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (popups.length) {
            popups[0].remove();
          }

          popup.addTo(map);
        });

        // CLICK OUT OF THE DEVICE
        map.on("click", (event) => {

          if (this.user.admin == 0 && event.originalEvent.ctrlKey) {
            let coordenates =
            {
              lat: event.lngLat.lat,
              lng: event.lngLat.lng
            }

            navigator.clipboard.writeText(coordenates.lat + ' ' + coordenates.lng)
              .then(() => {
              })
              .catch((error) => {
                window.emitter.emit('changeLoaderStatus', false, 'orbit')

                if(error.response && error.response.status && error.response.status == 429){
                  window.alert( this.$t('text.Api attempts'));
                }else{
                  console.error('Error al copiar el texto: ', error);
                  return "Failed!", error;
                }

              });          }

          if (this.screen_width >= 768 &&
            document.getElementById("sidemenu").clientWidth != "0" &&
            (this.charts[0] != undefined)
          ) {
            this.hide();
          }
          this.showlist = false;
        });

        // HOVER THE DEVICE
        map.on("mouseenter", service.name, (e) => {
          leaverdevices = 0;

          const description = e.features[0].properties.description;

          if (this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth == "0") {
            this.id_open = e.features[0].properties.id;
          }

          coordinates = this.coordinatesFromMarker(e);

          popup = new mapboxgl.Popup()
            .setLngLat(coordinates)
            .setHTML(description);

          if (!(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")) {
            // Obtener el ultimo dato
            this.getLastData(e.features[0].properties.id).then((response) => {

              if (response.type_name == null) {
                if (document.getElementsByClassName("mapboxgl-popup").length) {       
                  document.getElementById("popupDate_" + e.features[0].properties.id).innerHTML = 'Undefined';
                  document.getElementById("popupLastflow_" + e.features[0].properties.id).innerHTML = 'Undefined';
                  //this.hide();
                }
                console.error(this.$t("general.The device")+' "' + e.features[0].properties.name + '" (' + e.features[0].properties.id + ') ' + this.$t("text.has not sent any data yet"))
              } else if (response.type_name == 'MODBUS') { 

                const date = this.formatDate(response.dates[0], 0, false);

                if (document.getElementsByClassName("mapboxgl-popup").length && response){
                  document.getElementById("popupDate_" + e.features[0].properties.id).innerHTML = response.dates[0];
                }

              } else if (response.type_name == 'PARK') {

              } else if (response.type_name == 'PULSES'){
                if (document.getElementsByClassName("mapboxgl-popup").length && response.pulse) {                  
                  document.getElementById("popupDate_" + e.features[0].properties.id).innerHTML = response.pulse[2][0].date;
                  document.getElementById("popupLastflow_" + e.features[0].properties.id).innerHTML = response.pulse[2][0].values.flow;
                }
              }

            });
          }

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (
            popups.length &&
            !(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")
          ) {
            popups[0].remove();
          }

          if (!(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")) {
            popup.addTo(map);
          }

          map.getCanvas().style.cursor = "pointer";
        });

        // HOVER THE DEVICE
        $(document).on("mouseenter", ".mapboxgl-popup", () => {
          leaverpopup = 0;

          // Para los botones del POPUP
          if (document.getElementById("btnProfile") != null)
            document
              .getElementById("btnProfile")
              .addEventListener("click", () => {                
                document.getElementById('LinkProfile_' + this.id_open).click();
              });

          if (document.getElementById("btnReadings") != null)
            document
              .getElementById("btnReadings")
              .addEventListener("click", () => {
                document.getElementById('LinkReadings_' + this.id_open).click();
              });

          if (document.getElementById("btnPressureTransients") != null)
            document
              .getElementById("btnPressureTransients")
              .addEventListener("click", () => {
                document.getElementById('LinkPressureTransients_' + this.id_open).click();
              });
          
        });

        // UNHOVER THE DEVICE
        map.on("mouseleave", service.name, (e) => {
          popups = document.getElementsByClassName("mapboxgl-popup");

          setTimeout(() => {
            leaverdevices = 1;
            if (
              !(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0") &&
              leaverdevices * leaverpopup == 1 &&
              popups[0]
            ) {
              popups[0].remove();
            }
          }, 100);

          map.getCanvas().style.cursor = "";
        });

        // Layer cluster
        map.addLayer({
          id: "cluster_" + service.name,
          type: "symbol",
          source: service.name,
          filter: ["has", "point_count"],
          layout: {
            "icon-image": "cluster_marker",
            "icon-allow-overlap": true,
            "icon-anchor": "bottom",
            "icon-size": 1,
            "text-field": "{point_count_abbreviated}",
            "text-font": ["Roboto Bold"],
            "text-size": 11,
            "text-allow-overlap": true,
            "text-anchor": "bottom",
            "text-offset": [0, -1],
          },
          paint: {
            "text-color": "#FFFAFA",
          },
        });

        // CLICK IN THE CLUSTER
        map.on("click", "cluster_" + service.name, (e) => {
          const features = map.queryRenderedFeatures(e.point, {
            layers: ["cluster_" + service.name],
          });

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (popups.length) {
            popups[0].remove();
          }

          const clusterId = features[0].properties.cluster_id;
          map
            .getSource(service.name)
            .getClusterExpansionZoom(clusterId, (err, zoom) => {
              if (err) {
                return;
              }
              map.flyTo({
                center: features[0].geometry.coordinates,
                zoom: zoom + 3,
              });
              this.showlist = true;
            });
        });

        // HOVER THE CLUSTER
        map.on("mouseenter", "cluster_" + service.name, (e) => {
          coordinates = this.coordinatesFromMarker(e);

          popup = new mapboxgl.Popup().setLngLat(coordinates);

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (
            popups.length &&
            !(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")
          ) {
            popups[0].remove();
          }

          if (!(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0")) {
            popup.addTo(map);
          }

          map.getCanvas().style.cursor = "pointer";
        });

        // UNHOVER THE CLUSTER
        map.on("mouseleave", "cluster_" + service.name, (e) => {
          map.getCanvas().style.cursor = "";
        });
      });

      // UNHOVER THE POPUP
      $(document).on("mouseleave", ".mapboxgl-popup", () => {
        popups = document.getElementsByClassName("mapboxgl-popup");

        setTimeout(() => {
          leaverpopup = 1;
          if (
            !(this.screen_width >= 768 && document.getElementById("sidemenu").clientWidth != "0") &&
            leaverdevices * leaverpopup == 1 &&
            popups[0]
          ) {
            popups[0].remove();
          }
        }, 100);
      });

      // RENDERS MAP
      var rendercounter = 0;

      map.on("render", () => {
        var temp1 = rendercounter + 1;

        setTimeout(() => {
          if (temp1 == rendercounter) {
            this.isOnScreen(map);
            this.searchfilter();
            this.directAccessList(map);
          }
        }, 12);

        rendercounter++;
      });

      var bounds = new mapboxgl.LngLatBounds();
      this.MapFitBounds(500, map, bounds);
    });

    // After the last frame rendered before the map enters an "idle" state.
    map.on("idle", () => {
      // Enumerate ids of the layers.
      let toggleableLayerIds = [];
      let toggleableLayerImgs = [];

      this.group_service.forEach((service, index) => {
        toggleableLayerIds.push(service.name);
        switch (service.tag) {
          case 'DOM': toggleableLayerImgs.push({ 'Domiciliario' : 'domiciliaryLang' }); break;
          case 'DIS': toggleableLayerImgs.push({ 'Distribución' : 'distributionLang' }); break;
          case 'PRE': toggleableLayerImgs.push({ 'Control de presión' : 'pressureLang' }); break;
          case 'WQ': toggleableLayerImgs.push({ 'Calidad de agua' : 'waterQualityLang' }); break;
          case 'WL': toggleableLayerImgs.push({ 'Nivel de agua' : 'waterLevelLang' }); break;
        }
      });

      for (const id of toggleableLayerIds) {
        // Skip layers that already have a button set up.
        if (document.getElementById(id)) {
          continue;
        }

        // Create a link.
        const link = document.createElement("a");
        link.id = id;
        link.href = "#";
        //link.textContent = id;     
        
        const matchingObject = toggleableLayerImgs.find(obj => obj.hasOwnProperty(id));

        link.innerHTML = '<img class="h-4" src="/assets/images/markers/'+ matchingObject[id] + '.png" /><a>' + id + '</a>';

        link.className = "active inline-flex gap-1 justify-start items-center";

        // Show or hide layer when the toggle is clicked.
        link.onclick = (e) => {
          const clickedLayer = id//(link.textContent).toLowerCase();
          e.preventDefault();
          e.stopPropagation();

          var visibility = map.getLayoutProperty(clickedLayer, "visibility");

          // Toggle layer visibility by changing the layout object's visibility property.
          if (visibility === "visible") {
            map.setLayoutProperty(clickedLayer, "visibility", "none");
            map.setLayoutProperty('cluster_' + clickedLayer, "visibility", "none");
            link.className =
              "inline-flex gap-1 justify-start items-center text-gray-medium";
          } else {
            link.className =
              "active inline-flex gap-1 justify-start items-center";
            map.setLayoutProperty(clickedLayer, "visibility", "visible");
            map.setLayoutProperty('cluster_' + clickedLayer, "visibility", "visible");
          }

        };

        //if(this.screen_width >= 768){
          const layers = document.getElementById("legend");
          layers.appendChild(link);
        //}
      }
    });

    this.map = map;

  },

  methods: {

    directAccessList(map) {
      let device_list_item = document.querySelectorAll(".device_list_item");

      for (var marker = 0; marker < device_list_item.length; marker++) {
        device_list_item[marker].addEventListener("click", (event) => {
          let popup, popups;

          var targetElement = event.target.id
            .replace(/list_device_/g, "")
            .split("_");

          const description =
            this.geojsonobj[targetElement[0]].features[targetElement[1]]
              .properties.description;
          const temp_id =
            this.geojsonobj[targetElement[0]].features[targetElement[1]]
              .properties.id;
          this.id_open = temp_id;

          var aux_e = {};
          aux_e["lngLat"] = [];
          aux_e.lngLat = {
            lng: parseFloat(
              this.geojsonobj[targetElement[0]].features[targetElement[1]]
                .geometry.coordinates[0]
            ),
            lat: parseFloat(
              this.geojsonobj[targetElement[0]].features[targetElement[1]]
                .geometry.coordinates[1]
            ),
          };

          this.geojsonobj[targetElement[0]].features.forEach(
            (feature, index) => {
              if (feature.properties.id == temp_id) {
                marker = index;
              }
            }
          );

          this.show(aux_e, temp_id, map);

          let coordinates = [
            this.geojsonobj[targetElement[0]].features[targetElement[1]]
              .geometry.coordinates[0],
            this.geojsonobj[targetElement[0]].features[targetElement[1]]
              .geometry.coordinates[1],
          ];

          popup = new mapboxgl.Popup()
            .setLngLat(coordinates)
            .setHTML(description);

          popups = document.getElementsByClassName("mapboxgl-popup");

          if (popups.length) {
            popups[0].remove();
          }

          popup.addTo(map);
        });
      }
    },

    isOnScreen(map) {
      if (document.getElementById("device_list"))
        document.getElementById("device_list").innerHTML = "";

      this.group_service.forEach((service, index) => {

        if(this.geojsonobj[service.name].features){
          this.geojsonobj[service.name].features.forEach((feature) => {
            if (
              map.getBounds()._sw.lng <= feature.geometry.coordinates[0] &&
              feature.geometry.coordinates[0] <= map.getBounds()._ne.lng &&
              map.getBounds()._sw.lat <= feature.geometry.coordinates[1] &&
              feature.geometry.coordinates[1] <= map.getBounds()._ne.lat
            ) {
              feature.properties.insidescreen = true;
            } else {
              feature.properties.insidescreen = false;
            }
          });

          for (
            var marker = 0;
            marker < this.geojsonobj[service.name].features.length;
            marker++
          ) {
            if(document.getElementById("device_list")){
              document.getElementById("device_list").innerHTML +=
                `<li><a class='device_list_item hover:text-white-snow cursor-pointer' id="list_device_` +
                service.name +
                `_` +
                marker +
                `">` +
                this.geojsonobj[service.name].features[marker].properties.name +
                `</a></li>`;
            }
          }
        
        }
      });
    },

    searchfilter() {

      if(this.screen_width >= 768){

        let i = 0;

        this.group_service.forEach((service, index) => {
          var input,
            filter,
            ul,
            li,
            Name,
            id,
            textId,
            txtValueName,
            geojsonobj = this.geojsonobj[service.name];

          //text input
          input = document.getElementById("searchbar");

          //text to filter
          filter = input.value.toUpperCase();

          ul = document.getElementById("device_list");
          li = ul.getElementsByTagName("li");


          if(geojsonobj.features){
            geojsonobj.features.forEach((feature) => {
              // id search
              id = feature.properties.id;
              textId = id.toString();

              // name search
              Name = feature.properties.name;
              txtValueName = Name.toString();

              if (filter != "") {
                if (
                  txtValueName.toUpperCase().indexOf(filter) > -1 ||
                  filter == "ALL" ||
                  textId.indexOf(filter) > -1
                ) {
                  li[i].style.display = "";
                } else {
                  li[i].style.display = "none";
                }
              } else {
                if (feature.properties.insidescreen == true && this.showlist) {
                  li[i].style.display = "";
                } else {
                  li[i].style.display = "none";
                }
              }

              i++;
            });
          }
        });
      }
    },

    MapFitBounds(timeout, map, bounds) {
      setTimeout(() => {
        //bounds = new mapboxgl.LngLatBounds();
        this.group_service.forEach((service, index) => {

          if(this.geojsonobj[service.name].features){
            this.geojsonobj[service.name].features.forEach((feature) => {
              bounds.extend(feature.geometry.coordinates);
            });
          }
        });

        map.fitBounds(
          bounds,
          { padding: { top: 3, bottom: 3, left: 5, right: 5 } },
          { duration: 0 }
        );
      }, timeout);
    },

    // gets the coordenates from the device, cluster, etc we are clicking or hovering
    coordinatesFromMarker(e) {
      const coordinates = e.features[0].geometry.coordinates.slice();

      while (Math.abs(e.lngLat.lng - coordinates[0]) > 180) {
        coordinates[0] += e.lngLat.lng > coordinates[0] ? 360 : -360;
      }

      return coordinates;
    },

    show(e, id, map) {

      this.hide();
      var marker;

      //id
      this.group_service.forEach((service, index) => {
        if(this.geojsonobj[service.name].features){
          this.geojsonobj[service.name].features.forEach((feature, index) => {
            if (feature.properties.id == id) {
              marker = index;
              if(this.screen_width >= 768)
                this.getfulldata(feature.properties.id);
            }
          });
        }
      });
      if(document.getElementById("sidemenu")){
        document.getElementById("sidemenu").style.width = "20rem";
        document.getElementById("sidemenu").style.padding =
        "0.5rem 0.5rem 0 0.5rem";
      }

      // to point the device in the center of the map (sidemenu open)
      var metersPerPx =
        (156543.03392 * Math.cos((map.getCenter().lat * Math.PI) / 180)) /
        Math.pow(2, map.getZoom());

      if(this.screen_width >= 768)
        e.lngLat.lng += metersPerPx * 0.0009375;

      map.flyTo({
        center: e.lngLat,
        essential: true,
      });
    },

    hide() {
      if(document.getElementById("sidemenu")){  
        document.getElementById("sidemenu").style.width = "0";
        document.getElementById("sidemenu").style.padding = "0";
      }
      this.charts = new Array(10);
    },

    getfulldata(id) {
      this.getData(id).then((response) => {
        response.id = id;
        this.setDeviceData(response);
      });
    },

    setDeviceData(response) {

      this.charts = new Array(10);
      let highcharts_charts = new Array();

      this.charts_size = new Array();
      this.charts_text = new Array();
      this.charts_height = new Array();

      if(document.getElementById("sidemenu_content_before") && document.getElementById("sidemenu_content_after")){
        document.getElementById("sidemenu_content_before").innerHTML = "";
        document.getElementById("sidemenu_content_after").innerHTML = "";
      }

      const Highcharts = require("highcharts");

      if (response.type_name == null) {
        this.hide();

        window.emitter.emit(
            'AppLayout_alert', 
            true, 
            '', 
            this.$t('text.No data found for the selected time period'), 
            '18rem'
        )

      }else if (response.type_name == "PULSES") {

        if (document.getElementsByClassName("mapboxgl-popup").length && response.pulse) {                  
          document.getElementById("popupDate_" + response.id).innerHTML = response.pulse[2][0].date;
          document.getElementById("popupLastflow_" + response.id).innerHTML = response.pulse[2][0].values.flow;
        }

        let Chart = {
          chart: {
            backgroundColor: 'transparent',
            spacingTop: 15
          },
          accessibility: {
            enabled: false
          },
          title: {
            text: ''
          },
          xAxis: {
            //tickPositions: [0, night_start, 23],
            tickPositions: [0, 5, 11, 17, 23],
            categories: this.lastdayh,
            visible: true,
            labels: {
              style: {
                color: 'rgba(200, 200, 200, 0.9)'
              }
            },
          },
          yAxis: {
            title: {
              text: ''
            },
            labels: {
              style: {
                color: 'rgba(200, 200, 200, 0.9)'
              }
            },
            visible: true
          },
          exporting: {
            enabled: false
          },
          credits: {
            enabled: false
          },
          legend: {
            enabled: false,
          },
          tooltip: {
            //headerFormat: '<p style="font-size:12px; line-height: 12px; color:#b8bec0">{point.key} </p><br/>',
            //xDateFormat: '%Y-%m-%d  %H:%M:%S',
            shared: true,
            split: false,
            backgroundColor: 'rgba(7,82,103,0.95)',
            borderRadius: 12,
            borderWidth: 1,
            borderColor: 'rgba(250, 250, 250, 0.65)',
            padding: 6,
            formatter: function () {
              var s = '<b>' + '<p style="font-size:10px; line-height: 10px; font-weight: 300; color:rgba(250, 250, 250, 0.95)">' + this.x + ' → </p>';
              $.each(this.points, function () {
                s += '<p style="font-size:10px; line-height: 10px; font-weight: 400; color:' + this.series.color + '">' + this.y.toFixed(3) +
                  ' ' + this.series.userOptions.tooltip.valueSuffix + '</p><br/>';
              });


              return s;
            },
          },
        };

        var flowdata25h = new Array(25);
        var pressuredata25h = new Array(25);
        var flowtendence = new Array(168);
        var pressuretendence = new Array(168);

        response.pulse[2].forEach((data, index) => {
          if (index < 25) flowdata25h[index] = data.values.consumption;
          if (index >= 24) flowtendence[index - 24] = data.values.consumption;
        });

        if (response.pressure) {
          response.pressure.forEach(function callback(data, index) {
            if (index < 25) pressuredata25h[index] = parseFloat(data.values.avg);
            if (index >= 24) pressuretendence[index - 24] = parseFloat(data.values.avg);
          });
        }

        var flowdata24h = flowdata25h.slice(0, 24);
        var flowincrease24h = new Array(24);

        var pressuredata24h = pressuredata25h.slice(0, 24);
        var pressureincrease24h = new Array(24);

        //var event = new Date((new Date((response.pulse[2][0].date))).getTime() - timezone * 1000 * 60 * 60);

        const firsthourTEMP = (response.pulse[2][0].date).substring(11, 13);

        var night_start, night_end;
        var lastdayh = new Array(24);

        for (var i = 0; i < 24; i++) { // we get the index in lastdayh[] that corresponds with 00h and 06h
          lastdayh[i] = (firsthourTEMP - 23 + i);
          if (lastdayh[i] < 0) lastdayh[i] += 24;
          if (lastdayh[i] == 0) night_start = i;
          else if (lastdayh[i] == 6) night_end = i;
          lastdayh[i] += 'h';
        };

        if (night_start < night_end) {
          Chart.xAxis.plotBands = [
            {
              from: night_start,
              to: night_end,
              color: 'rgba(25,25,25,0.50)',
            }
          ]
        } else {
          Chart.xAxis.plotBands = [
            {
              from: 0,
              to: night_end,
              color: 'rgba(25,25,25,0.50)',
            }, {
              from: night_start,
              to: lastdayh.length - 1,
              color: 'rgba(25,25,25,0.50)',
            }
          ]
        }

        var flowdata24h_tendence = new Array(24);
        var pressuredata24h_tendence = new Array(24);

        for (var i = 0; i < flowdata24h_tendence.length; i++) {
          flowdata24h_tendence[i] = (flowtendence[i] + flowtendence[i + 24] + flowtendence[i + 48] + flowtendence[i + 72] + flowtendence[i + 96] + flowtendence[i + 120] + flowtendence[i + 144]) / 7;
        }

        for (var i = 0; i < pressuredata24h_tendence.length; i++) {
          pressuredata24h_tendence[i] = (pressuretendence[i] + pressuretendence[i + 24] + pressuretendence[i + 48] + pressuretendence[i + 72] + pressuretendence[i + 96] + pressuretendence[i + 120] + pressuretendence[i + 144]) / 7;
        }

        Chart.xAxis.categories = lastdayh;

        Chart.yAxis.tickPositions = [0, parseFloat((Math.max(...flowdata24h)).toFixed(3))];

        Chart.tooltip.valueSuffix = 'm³';

        for (var i = 0; i < 24; i++) { // getting the difference between last 25 hours and last 24 hours to get flow variations
          flowincrease24h[i] = flowdata24h[i] - flowdata25h[i + 1];
        };

        for (var i = 0; i < 24; i++) { // getting the difference between last 25 hours and last 24 hours to get flow variations
          pressureincrease24h[i] = pressuredata24h[i] - pressuredata25h[i + 1];
        };

        if (!response.pressure) {

          this.charts[0] = {
            name: this.$t("water." + "lastConsumptionsLang"),
          };

          Chart.series = [{
            name: this.$t("water." + "lastConsumptionsLang"),
            type: 'areaspline',
            data: flowdata24h.reverse(),
            color: "#05fff3",
            lineWidth: 0,
            opacity: 0.825,
            marker: {
              enabled: false
            },
            tooltip: {
              valueSuffix: 'm³'
            }
          }, {
            name: this.$t("water." + "consumptionsTendenceLang"), // trans("water."+"consumptionsTendenceLang"),
            type: 'spline',
            data: flowdata24h_tendence.reverse(),
            color: "#6DFF33",
            enableMouseTracking: false,
            lineWidth: 0.75,
            dashStyle: 'dash',
            marker: {
              enabled: false
            },
            tooltip: {
              valueSuffix: 'm³'
            }
          }
          ];

          if(document.getElementById("chart_0"))
            highcharts_charts.push(Highcharts.chart("chart_0", Chart));

          this.charts[1] = {
            name: this.$t("water." + "consumptionsVariationLang"),
          };

          Chart.yAxis.tickPositions = [parseFloat((Math.min(...flowincrease24h)).toFixed(3)), 0, parseFloat((Math.max(...flowincrease24h)).toFixed(3))];

          Chart.tooltip.valueSuffix = 'm³';

          Chart.series = [{
            name: this.$t("water." + "consumptionsVariationLang"),
            type: 'column',
            data: flowincrease24h.reverse(),
            color: "#05fff3",
            pointWidth: 5,
            borderWidth: 0,
            tooltip: {
              valueSuffix: 'm³'
            }
          }];

          if(document.getElementById("chart_1"))
          highcharts_charts.push(Highcharts.chart("chart_1", Chart));

        } else {

          this.charts[0] = {
            name: this.$t("water." + "lastPressuresLang"),
          };

          Chart.yAxis.tickPositions = [parseFloat((Math.min(...pressuredata24h)).toFixed(2)), parseFloat((Math.max(...pressuredata24h)).toFixed(2))];

          Chart.tooltip.valueSuffix = 'bar';

          Chart.series = [{
            name: this.$t("water." + "lastPressuresLang"),
            type: 'spline',
            data: pressuredata24h.reverse(),
            color: "#05fff3",
            lineWidth: 1.5,
            zIndex: 1,
            marker: {
              enabled: false
            },
            tooltip: {
              valueSuffix: 'bar'
            }
          }, {
            name: this.$t("water." + "pressuresTendenceLang"),
            type: 'spline',
            data: pressuredata24h_tendence.reverse(),
            color: "#6DFF33",
            enableMouseTracking: false,
            lineWidth: 0.75,
            dashStyle: 'dash',
            marker: {
              enabled: false
            },
            tooltip: {
              valueSuffix: 'bar'
            }
          }];

          if(document.getElementById("chart_0"))
          highcharts_charts.push(Highcharts.chart("chart_0", Chart));

          this.charts[1] = {
            name: this.$t("water." + "pressuresVariationLang"),
          };

          Chart.yAxis.tickPositions = [parseFloat((Math.min(...pressureincrease24h)).toFixed(3)), 0, parseFloat((Math.max(...pressureincrease24h)).toFixed(3))];

          Chart.tooltip.valueSuffix = 'bar';

          Chart.series = [{
            name: this.$t("water." + "pressuresVariationLang"),
            type: 'column',
            data: pressureincrease24h.reverse(),
            color: "#05fff3",
            pointWidth: 5,
            borderWidth: 0,
            tooltip: {
              valueSuffix: 'bar'
            }
          }];

          if(document.getElementById("chart_1"))
          highcharts_charts.push(Highcharts.chart("chart_1", Chart));

          this.charts[2] = {
            name: this.$t("water." + "lastConsumptionsLang"),
          };

          Chart.yAxis.tickPositions = [0, parseFloat((Math.max(...flowdata24h)).toFixed(3))];

          Chart.series = [{
            name: this.$t("water." + "lastConsumptionsLang"),
            type: 'areaspline',
            data: flowdata24h.reverse(),
            color: "#05fff3",
            lineWidth: 0,
            opacity: 0.825,
            marker: {
              enabled: false
            },
            tooltip: {
              valueSuffix: 'm³'
            }
          }, {
            name: this.$t("water." + "consumptionsTendenceLang"), // trans("water."+"consumptionsTendenceLang"),
            type: 'spline',
            data: flowdata24h_tendence.reverse(),
            color: "#6DFF33",
            enableMouseTracking: false,
            lineWidth: 0.75,
            dashStyle: 'dash',
            marker: {
              enabled: false
            },
            tooltip: {
              valueSuffix: 'm³'
            }
          }
          ];

          if(document.getElementById("chart_2"))
          highcharts_charts.push(Highcharts.chart("chart_2", Chart));

          this.charts[3] = {
            name: this.$t("water." + "consumptionsVariationLang"),
          };

          Chart.yAxis.tickPositions = [parseFloat((Math.min(...flowincrease24h)).toFixed(3)), 0, parseFloat((Math.max(...flowincrease24h)).toFixed(3))];

          Chart.tooltip.valueSuffix = 'm³';

          Chart.series = [{
            name: this.$t("water." + "consumptionsVariationLang"),
            type: 'column',
            data: flowincrease24h.reverse(),
            color: "#05fff3",
            pointWidth: 5,
            borderWidth: 0,
            tooltip: {
              valueSuffix: 'm³'
            }
          }];

          if(document.getElementById("chart_3"))
          highcharts_charts.push(Highcharts.chart("chart_3", Chart));

        }
        
      } else if (response.type_name == "MODBUS") {

        let sensor_num;

        if (response.data) {
          sensor_num = Object.keys(response.data).length;
        } else {
          sensor_num = 1;
        }

        let units = new Array(sensor_num);
        let names = new Array(sensor_num);
        let data = new Array(sensor_num);
        let dates;

        if (document.getElementsByClassName("mapboxgl-popup").length) {
          document.getElementById("popupDate_" + response.id).innerHTML  = this.formatDate(
            response.dates[0],
            0, false
          );
        }

        for (let sensor in response.data) {
          this.charts[sensor - 1] = {
            name:
              (response.units[sensor] != '') ? (response.names[sensor] + " (" + response.units[sensor] + ")") : (response.names[sensor]),
          };

          dates = new Array(response.data[sensor].length);
          data[sensor - 1] = new Array(response.data[sensor].length);
          names[sensor - 1] = response.names[sensor];
          units[sensor - 1] = response.units[sensor];

          response.data[sensor].forEach((value, index) => {
            dates[index] = new Date(
              (response.dates[index] + " GMT")
            ).getTime();
            data[sensor - 1][index] = parseFloat(value);
          });
        }

        data.forEach((sensor, index) => {
          let Chart = this.Chart;

          let highcharts_data = new Array(sensor.length);

          sensor.forEach((measure, index) => {
            highcharts_data[index] = [dates[index], measure];
          });

          Chart.yAxis.tickPositions = [
            parseFloat(Math.min(...sensor)),
            parseFloat(Math.max(...sensor)),
          ];

          Chart.tooltip.valueSuffix = units[index];

          Chart.series = [
            {
              name: names[index],
              type: "spline",
              data: highcharts_data.reverse(),
              color: "#05fff3",
              lineWidth: 1.5,
              zIndex: 1,
              marker: {
                enabled: false,
              },
              tooltip: {
                valueSuffix: units[index],
              },
            },
          ];
          
          if(document.getElementById("chart_" + index))
          highcharts_charts.push(Highcharts.chart("chart_" + index, Chart));
        });

      } else if (response.type_name == "PARK") {

      }

      setTimeout(() => {

        for (let i_chart = 0; i_chart < highcharts_charts.length; i_chart++) {
          if (highcharts_charts[i_chart] != undefined) {
            highcharts_charts[i_chart].reflow();
          }
        }
      }, 200);
    }
  },
};
</script>



